/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_BLAKE3_H
#define LIMB_BLAKE3_H

#include <stddef.h> /* size_t */
#include <limb/int.h>

#define BLAKE3_KEY_LEN 32
#define BLAKE3_OUT_LEN 32
#define BLAKE3_BLOCK_LEN 64
#define BLAKE3_CHUNK_LEN 1024
#define BLAKE3_MAX_DEPTH 54

/* This struct is a private implementation detail. It has to be here because
 * it's part of blake3_hasher below. */
typedef struct {
  u32 cv[8];
  u64 chunk_counter;
  u8 buf[BLAKE3_BLOCK_LEN];
  u8 buf_len;
  u8 blocks_compressed;
  u8 flags;
} blake3_chunk_state;

typedef struct {
  u32 key[8];
  blake3_chunk_state chunk;
  u8 cv_stack_len;
  /* The stack size is MAX_DEPTH + 1 because we do lazy merging. For example,
   * with 7 chunks, we have 3 entries in the stack. Adding an 8th chunk
   * requires a 4th entry, rather than merging everything down to 1, because we
   * don't know whether more input is coming. This is different from how the
   * reference implementation does things.
   */
  u8 cv_stack[(BLAKE3_MAX_DEPTH + 1) * BLAKE3_OUT_LEN];
} blake3_ctx;

extern void blake3_init(void *ctx);
extern void blake3_update(const void *msg, size_t mlen, void *ctx);
extern void blake3_final(void * restrict md, void *ctx);

#endif /* LIMB_BLAKE3_H */
