/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_BUFFER_SHLDATA_H
#define LIMB_BUFFER_SHLDATA_H

#include <sys/types.h>
#include <limb/stralloc.h>
#include <limb/buffer.h>
#include <limb/shldata.h>

struct buffer_shldata_ctx {
    stralloc sa;
    shldata_ctx sd;
};

#define BUFFER_SHLDATA_ZERO     { .sa = STRALLOC_ZERO }

typedef struct buffer_shldata_ctx buffer_shldata_ctx;

extern void buffer_shldata_free(buffer_shldata_ctx *ctx);

/* Writing */

extern ssize_t buffer_shldata_putinit(buffer *b, const char *pwd, size_t plen, unsigned algo, unsigned iter, size_t len, buffer_shldata_ctx *ctx);
extern ssize_t buffer_shldata_put(buffer *b, char *data, size_t dlen, int inplace, buffer_shldata_ctx *ctx);
extern ssize_t buffer_shldata_putfinal(buffer *b, buffer_shldata_ctx *ctx);

/* Reading */

extern int buffer_shldata_getinit_sa(buffer *b, const char *pwd, size_t plen, unsigned *algo, unsigned *iter, stralloc *sa, buffer_shldata_ctx *ctx);
extern int buffer_shldata_getinit(buffer *b, const char *pwd, size_t plen, unsigned *algo, unsigned *iter, buffer_shldata_ctx *ctx);
#define buffer_shldata_datasize(ctx)     shldata_datasize(&(ctx)->sd)
extern ssize_t buffer_shldata_get(buffer *b, char *dst, size_t dlen, buffer_shldata_ctx *ctx);
extern int buffer_shldata_getfinal_sa(buffer *b, stralloc *sa, buffer_shldata_ctx *ctx);
extern int buffer_shldata_getfinal(buffer *b, buffer_shldata_ctx *ctx);

#endif /* LIMB_BUFFER_SHLDATA_H */
