/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_BUFFER_H
#define LIMB_BUFFER_H

#include <skalibs/buffer.h>
#include <limb/hex.h> /* HEX_* */
#include <limb/int.h>
#include <limb/siovec.h>

/* special values usable as strings for buffer_putmsg() */
#define PUTMSG_FLUSH        ((void *) 1)    /* flush buffer */
#define PUTMSG_TOGGLE_ESC   ((void *) 2)    /* toggle escaping */
#define PUTMSG_ESC          ((void *) 3)    /* toggle escaping & add quote */
#define PUTMSG_UINT_NEXT    ((void *) 4)    /* next string is an uint */
#define PUTMSG_INT_NEXT     ((void *) 5)    /* next string is an int */
#define PUTMSG_ERR_NEXT     ((void *) 6)    /* add strerror(next) */
#define PUTMSG_LEN_NEXT     ((void *) 7)    /* next strings are data, dlen */
#define PUTMSG_HEX_NEXT     ((void *) 8)    /* next strings are data, dlen */
#define PUTMSG_DUMP_NEXT    ((void *) 9)    /* next strings are data, dlen */

#define PUTMSG_ASUINT(n)    ((const char *) (uintptr_t) (n))
#define PUTMSG_ASINT(n)     ((const char *) ( intptr_t) (n))

#define PUTMSG_UINT(n)      PUTMSG_UINT_NEXT, PUTMSG_ASUINT(n)
#define PUTMSG_INT(n)       PUTMSG_INT_NEXT,  PUTMSG_ASINT(n)
#define PUTMSG_ERR(n)       PUTMSG_ERR_NEXT,  PUTMSG_ASUINT(n)
#define PUTMSG_LEN(d,l)     PUTMSG_LEN_NEXT,  d, PUTMSG_ASUINT(l)
#define PUTMSG_HEX(d,l)     PUTMSG_HEX_NEXT,  d, PUTMSG_ASUINT(l)
#define PUTMSG_DUMP(d,l)    PUTMSG_DUMP_NEXT, d, PUTMSG_ASUINT(l)

#define ESC                 PUTMSG_ESC
#define PMUINT(n)           PUTMSG_UINT(n)
#define PMINT(n)            PUTMSG_INT(n)
#define PMERR(n)            PUTMSG_ERR(n)
#define PMLEN(d,l)          PUTMSG_LEN(d,l)
#define PMHEX(d,l)          PUTMSG_HEX(d,l)
#define PMDMP(d,l)          PUTMSG_DUMP(d,l)

/* Writing */

extern void buffer_putmsg(buffer *b, const char * const *as, unsigned int n);
extern int buffer_putescall(buffer *b, const char *s, size_t len, size_t *w, size_t *r);
extern ssize_t buffer_putesc(buffer *b, const char *s, size_t len);
extern ssize_t buffer_putescs(buffer *b, const char *s);
extern ssize_t buffer_puthex(buffer *b, const void *data, size_t dlen, int options);

typedef ssize_t (*base_fmt_fn) (char *dst, const char *data, size_t dlen, int pad);
extern ssize_t buffer_putbase(buffer *b, const char *data, size_t dlen, base_fmt_fn basefmt, size_t bin, size_t bout, int pad);
#define buffer_putbase32(b,d,l,p)   buffer_putbase(b, d, l, base32, 40, 64, p)
#define buffer_putbase64(b,d,l,p)   buffer_putbase(b, d, l, base64, 42, 56, p)

extern ssize_t buffer_puthdr(buffer *b, u32 magic, u64 ver);
extern ssize_t buffer_patrim_put(buffer *b, u64 id, u64 u, const char *data);
extern ssize_t buffer_patrim_putv(buffer *b, u64 id, const struct iovec v[], unsigned int n);

/* Reading */

extern int buffer_getuptoc(buffer *b, char *dst, size_t len, char c, size_t *got);

extern int buffer_gethdr(buffer *b, u32 *magic, u64 *ver);
extern int buffer_patrim_get(buffer *b, u64 *id, u64 *u);

#endif /* LIMB_BUFFER_H */
