/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_DJBUNIX_H
#define LIMB_DJBUNIX_H

#include <skalibs/djbunix.h>
#include <skalibs/unix-transactional.h>
#include <limb/posixplz.h>
#include <limb/siovec.h>
#include <limb/stralloc.h>

typedef int (*open_fn) (const char *file);
typedef int (*openat_fn) (int bfd, const char *file);

extern int open_b(open_fn fn, const char *file);
extern int open_atb(openat_fn fn, int bfd, const char *file);

extern int openb_readat(int fd, const char *file);
extern int openbc_readat(int fd, const char *file);

extern int open_writeb(const char *file);
extern int openb_write(const char *file);
extern int openb_writeat(int fd, const char *file);
extern int openc_writeb(const char *file);
extern int openbc_write(const char *file);
extern int openbc_writeat(int fd, const char *file);

extern int open_createat(int fd, const char *file);
extern int openc_createat(int fd, const char *file);

extern int open_exclat(int fd, const char *file);
extern int openc_exclat(int fd, const char *file);

#define open_tmp(tmp)           mkfiletemp(tmp, (create_func *) (void (*)(void)) open_excl, 0, NULL)
#define open_tmpat(fd,tmp)      mkfiletempat(fd, tmp, (createat_fn) (void (*)(void)) open_exclat, 0, NULL)

extern int open_parsed_name(const char *name, open_fn open);

#define read_close(dst,dlen,fd) readnclose(fd, dst, dlen)
extern int sa_readmax(stralloc *sa, int fd, size_t max);
#define sa_read(sa,fd)          sa_readmax(sa, fd, 0)

extern ssize_t open_read_closeat(char *dst, size_t dlen, int bfd, const char *file);
extern int open_saread_closeat(stralloc *sa, int bfd, const char *file);
extern int open_write_closeat(int fd, const char *file, const char *data, size_t dlen);
extern int open_writev_closeat(int fd, const char *file, const struct iovec *v, unsigned n);

#define open_read_close(dst,dlen,file)      open_read_closeat(dst, dlen, AT_FDCWD, file)
#define open_saread_close(sa,file)          open_saread_closeat(sa, AT_FDCWD, file)
#define open_write_close(file,data,dlen)    open_write_closeat(AT_FDCWD, file, data, dlen)
#define open_writev_close(file,v,n)         open_writev_closeat(AT_FDCWD, file, v, n)

extern int fd_same(int fd1, int fd2);

extern int fd_mkdirpat(int fd, const char *name, mode_t mode);
extern int fd_mkdirp(const char *name, mode_t mode);
extern int mkdirpat(int fd, const char *name, mode_t mode);
extern int mkdirp(const char *name, mode_t mode);

extern int rm_rfat(int fd, const char *name);
extern int rm_rf_tmpat(int fd, const char *name, stralloc *sa);
extern int rm_rf_in_tmpat(int fd, stralloc *sa, size_t offset);
extern int rmstarat(int fd, const char *name);
extern int rmstar_tmpat(int fd, const char *name, stralloc *sa);
extern int rmstar_in_tmpat(int fd, stralloc *sa, size_t offset);

extern int sa_readlinkat(stralloc *sa, int fd, const char *file);
extern int sa_readlinkat0(stralloc *sa, int fd, const char *file);
extern int sa_realpathat(stralloc *sa, int fd, const char *file);
extern int sa_lsat(stralloc *sa, int fd, const char *name, size_t *maxlen);

#define sa_readlink(sa,name)            sa_readlinkat(sa, AT_FDCWD, name)
#define sa_readlink0(sa,name)           sa_readlinkat0(sa, AT_FDCWD, name)
#define sa_realpath(sa,name)            sarealpath(sa, name)
#define sa_ls(sa,name,maxlen)           sa_lsat(sa, AT_FDCWD, name, maxlen)

#endif /* LIMB_DJBUNIX_H */
