/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_HASHER_H
#define LIMB_HASHER_H

#include <stddef.h> /* size_t */
#include <limb/int.h>

typedef void (*hasher_init)   (void *ctx);
typedef void (*hasher_update) (const void *msg, size_t mlen, void *ctx);
typedef void (*hasher_final)  (void *digest, void *ctx);

typedef struct hasher hasher;

struct hasher {
    size_t hlen;
    size_t blen;
    hasher_init init;
    hasher_update update;
    hasher_final final;
    u8 ctx[];
};

#define hinit(h)            h->init(h->ctx)
#define hupdate(d,l,h)      h->update(d, l, h->ctx)
#define hfinal(o,h)         h->final(o, h->ctx)

extern void hasher_hash(void *digest, const void *msg, size_t mlen, hasher *hasher);

/* supported algorithms/hashers */
/* XXX make sure to preserve order in *algos[] in hashers.c */
enum {
    ALGO_SHA1 = 0,
    ALGO_SHA256,
    ALGO_SHA512,
    ALGO_SHA3_224,
    ALGO_SHA3_256,
    ALGO_SHA3_384,
    ALGO_SHA3_512,
    ALGO_BLAKE3,
    NB_ALGOS
};

/* names of said algos, in the same order */
extern const char * const algos[NB_ALGOS + 1];
extern hasher * const hashers[NB_ALGOS];

#endif /* LIMB_HASHER_H */
