/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_LOADOPT_H
#define LIMB_LOADOPT_H

#include <limb/direntry.h>
#include <limb/parseopt.h>
#include <limb/stralloc.h>

enum {
    /* private/internal */
    OPT_WAIT    = 0,
    OPT_DONE    = 1 << 0,
    OPT_SET     = 1 << 1,
    /* public * */
    OPT_SKIP    = 1 << 2,               /* don't load from file */
    OPT_REQ     = 1 << 3,               /* must be specified */
    OPT_SA      = 1 << 4,               /* copy into sa */
    OPT_ESC_    = 1 << 5,
    OPT_ESC     = OPT_ESC_ | OPT_SA,    /* + unesc if w/in quotes */
    OPT_PATH_   = 1 << 6,
    OPT_PATH    = OPT_PATH_ | OPT_ESC,  /* + ensure no NUL byte */
    OPT_FILE_   = 1 << 7,
    OPT_FILE    = OPT_FILE_ | OPT_PATH, /* + ensure no '/' */
    OPT_RPT     = 1 << 8,               /* for last argument only */
};

/* end of options, on to arguments */
#define LOADOPT_ARGUMENTS   { 0, 0, ARG_REQ,  OPT_DONE, 0, 0 }

/* to define required argument, optional argument */
#define ARGUMENT_REQ(a,f,id)    { 0, a, ARG_REQ, f, 0, id }
#define ARGUMENT_OPT(a,f,id)    { 0, a, ARG_OPT, f, 0, id }

/* end of arguments (more == "too many args" */
#define LOADOPT_DONE        { 0, 0, ARG_NONE, OPT_DONE, 0, 0 }
/* stop parsing here, any more args will not our call */
#define LOADOPT_STOP        { 0, 0, ARG_OPT,  OPT_DONE, 0, 0 }

/* opaque structure */
struct loadopt {
    struct parseopt po;
    DIR *confdir;
    size_t optoff;  /* public, when OPT_SA is used */
    u8 optflags[64];
    u16 loflags         : 4;    /* LOADOPT_* */
    u16 state           : 4;
    u16 left            : 7;
    /* public (read-only) */
    u16 from_file       : 1;
};

#define LOADOPT_ZERO        { PARSEOPT_ZERO, NULL, 0, { 0 }, 0, 0, 0, 0 }

#define LO_ARG(ctx)         (ctx)->po.arg
#define LO_OFF(ctx)         (ctx)->optoff
#define LO_IDX(ctx)         (ctx)->po.idx
#define LO_CUR(ctx)         (ctx)->po.cur
#define LO_FROMFILE(ctx)    (ctx)->from_file

extern int loadopt(stralloc *sa, int argc, const char **argv, const struct option *options,
                   int bfd, const char *confdir, unsigned int poflags, struct loadopt *ctx);

/* dont load (specified) options fro config file */
#define AUTOOPT_NOCONFIG(s,l,f,i)   OPTION_ARG_OPT( s, l, OPT_SKIP | f, i)
extern int loadopt_auto_noconfig(int idx, const struct option *options, struct loadopt *ctx);

#endif /* LIMB_LOADOPT_H */
