/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_OBUFFER_H
#define LIMB_OBUFFER_H

#include <limb/buffer.h>
#include <limb/int.h>

enum olevel {
    OLVL_SILENT     = 0,
    OLVL_QUIET      = 50,
    OLVL_NORMAL     = 100,
    OLVL_VERBOSE    = 150,
    OLVL_DEBUG      = 200,
    OLVL_MAXIMUM    = 250,
};

typedef struct obuffer obuffer;

/* opaque structure */
struct obuffer {
    buffer *b;
    obuffer *n;
    u8 lvl;
};

#define OBUFFER_INIT(buf,lvl)           { buf, NULL, lvl }

extern void obuffer_put(obuffer *obuf, u8 level, const char *s, size_t len);
extern void obuffer_putmsg(obuffer *obuf, u8 level, const char * const *as, unsigned int n);
#define obuffer_level(obuf)             (obuf)->lvl
#define obuffer_setlevel(obuf,level)    (obuf)->lvl = level

extern int obuffer_attach(obuffer *obuf, obuffer *to);
extern int obuffer_detach(obuffer *obuf);
#define obuffer_is_attached(obuf)       ((int) !!(obuf)->n)

extern u8 obuffer_parse_level(const char *str, size_t len);

extern obuffer obuffer_1_;
#define obuffer_1       (&obuffer_1_)

extern obuffer obuffer_2_;
#define obuffer_2       (&obuffer_2_)

#endif /* LIMB_OBUFFER_H */
