/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_OUTPUT_H
#define LIMB_OUTPUT_H

#include <errno.h>
#include <stddef.h> /* size_t */
#include <limb/gccattributes.h>
#include <limb/obuffer.h>

extern void obuffer_putmsgdie(obuffer *obuf, u8 level, const char * const str[], unsigned n, int exit) gccattr_noreturn;
extern void lila_dieversion(const char *version, const char *yearbeg, const char *yearend, const char *author, const char *url, const char *license, int is_lila) gccattr_noreturn;

extern void list_matches_full(obuffer *obuf, u8 level, const char *intro,
                              const char *prefix, const char *sep, const char *outro,
                              const char *str, size_t slen, int first,
                              const void *list_, size_t offset, size_t llen);
extern void list_matches(obuffer *obuf, u8 level, const char *intro,
                         const char *prefix, const char *sep, const char *outro,
                         const char *str, size_t slen, int first, const char **list);

extern const char *PROG;

#define outarray(...)       ((const char * const []) {__VA_ARGS__})
#define outalen(...)        (sizeof(outarray(__VA_ARGS__)) / sizeof(char *))

#define putmsg(b,l,...)     obuffer_putmsg(b, l, outarray(__VA_ARGS__), outalen(__VA_ARGS__))
#define outmsg(l,...)       putmsg(obuffer_1, l, __VA_ARGS__)
#define errmsg(l,...)       putmsg(obuffer_2, l, __VA_ARGS__)
#define dbgmsg(l,...)       outmsg(l, "[", __func__, "@", __FILE__, ":", PUTMSG_INT(__LINE__), "] ", __VA_ARGS__)

#define putmsgdie(e,b,l,...)obuffer_putmsgdie(b, l, outarray(__VA_ARGS__), outalen(__VA_ARGS__), e)
#define outmsgdie(e,l,...)  putmsgdie(e, obuffer_1, l, __VA_ARGS__)
#define errmsgdie(e,l,...)  putmsgdie(e, obuffer_2, l, __VA_ARGS__)

#define quiet(...)          outmsg(OLVL_QUIET, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define out(...)            outmsg(OLVL_NORMAL, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define verb(...)           outmsg(OLVL_VERBOSE, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define dbg(...)            dbgmsg(OLVL_DEBUG, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define dbgverb(...)        dbgmsg(OLVL_MAXIMUM, __VA_ARGS__, "\n", PUTMSG_FLUSH)

#define err(...)            errmsg(OLVL_NORMAL, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define errverb(...)        errmsg(OLVL_VERBOSE, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define sys(...)            errmsg(OLVL_NORMAL, __VA_ARGS__, ": ", PUTMSG_ERR(errno), "\n", PUTMSG_FLUSH)
#define sysverb(...)        errmsg(OLVL_VERBOSE, __VA_ARGS__, ": ", PUTMSG_ERR(errno), "\n", PUTMSG_FLUSH)

#define add(...)            outmsg(OLVL_NORMAL, __VA_ARGS__)
#define adde(...)           errmsg(OLVL_NORMAL, __VA_ARGS__)

#define quietdie(e,...)     outmsgdie(e, OLVL_QUIET, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define outdie(e,...)       outmsgdie(e, OLVL_NORMAL, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define verbdie(e,...)      outmsgdie(e, OLVL_VERBOSE, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define errdie(e,...)       errmsgdie(e, OLVL_NORMAL, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define errverbdie(e,...)   errmsgdie(e, OLVL_VERBOSE, __VA_ARGS__, "\n", PUTMSG_FLUSH)
#define sysdie(e,...)       errmsgdie(e, OLVL_NORMAL, __VA_ARGS__, ": ", PUTMSG_ERR(errno), "\n", PUTMSG_FLUSH)
#define sysverbdie(e,...)   errmsgdie(e, OLVL_VERBOSE, __VA_ARGS__, ": ", PUTMSG_ERR(errno), "\n", PUTMSG_FLUSH)

#define warn(...)           err(PROG, ": warning: ", __VA_ARGS__)
#define warnu(...)          warn("unable to ", __VA_ARGS__)
#define warnsys(...)        sys(PROG, ": warning: ", __VA_ARGS__)
#define warnusys(...)       warnsys("unable to ", __VA_ARGS__)

#define die(e, ...)         errdie(e, PROG, ": ", __VA_ARGS__)
#define dief(e, ...)        die(e, "fatal: ", __VA_ARGS__)
#define diefu(e, ...)       dief(e, "unable to ", __VA_ARGS__)
#define diesys(e, ...)      sysdie(e, PROG, ": ", __VA_ARGS__)
#define diefsys(e, ...)     diesys(e, "fatal: ", __VA_ARGS__)
#define diefusys(e, ...)    diefsys(e, "unable to ", __VA_ARGS__)

#define errhelp(u, h)       err("usage: ", PROG, " ", u, "\n\n", h)

#define dieusage(e, u)      errdie(e, "usage: ", PROG, " ", u)
#define diehelp(e, u, h)    errdie(e, "usage: ", PROG, " ", u, "\n\n", h)

#define diecmdusage(e, u, c)errdie(e, "usage: ", PROG, " ", u, (c)->name, " ", (c)->usage)
#define diecmdhelp(e, u, c) errdie(e, "usage: ", PROG, " ", u, (c)->name, " ", (c)->usage, "\n", (c)->desc, "\n\n", (c)->help)

#define dieversion(v , yb, ye, a, u, l)     lila_dieversion(v, yb, ye, a, u, l, 0)
#define liladieversion(v, yb, ye, a, u ,l)  lila_dieversion(v, yb, ye, a, u, l, 1)

#define retw(r, ...)        do { warn(__VA_ARGS__);        return r; } while (0)
#define retwu(r, ...)       do { warnu(__VA_ARGS__);       return r; } while (0)
#define retwsys(r, ...)     do { warnsys(__VA_ARGS__);     return r; } while (0)
#define retwusys(r, ...)    do { warnusys(__VA_ARGS__);    return r; } while (0)

#endif /* LIMB_OUTPUT_H */
