/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_PARSEOPT_H
#define LIMB_PARSEOPT_H

#include <limb/int.h>

enum {
    ARG_NONE = 0,   /* no argument */
    ARG_REQ,        /* argument required */
    ARG_OPT         /* argument optional */
};

/* to define options with no arg, optinal arg, required arg */
#define OPTION_ARG_NONE(s,l,f,i)    { s, l, ARG_NONE, f, 0, i }
#define OPTION_ARG_OPT(s,l,f,i)     { s, l, ARG_OPT,  f, 0, i }
#define OPTION_ARG_REQ(s,l,f,i)     { s, l, ARG_REQ,  f, 0, i }

/* last element in struct option[] to indicate the end */
#define OPTION_DONE                 { .flags = 1 }

/* special ID meaning to return shortopt instead */
#define OPTID_SHORTOPT              0

/* first value for user option.id */
#define OPTID_FIRST                 127

struct option {
    const char  shortopt;
    const char *longopt;
    u16 arg     :  2; /* ARG_* */
    u16 flags   : 10; /* OPT_* -- loadopt only */
    u16 _unused :  4;
    int id;
};

enum {
    PARSEOPT_STRICT     = 1 << 0,
    PARSEOPT_SILENT     = 1 << 1,
};

/* opaque structure -- use macros below to access its public members */
struct parseopt {
    const char *arg;
    int idx;
    u16 cur;
    u16 off;
};

#define PARSEOPT_ZERO       { NULL, 0, 0, 0 }

#define PO_ARG(ctx)         (ctx)->arg
#define PO_IDX(ctx)         (ctx)->idx
#define PO_CUR(ctx)         (ctx)->cur

extern int parseopt(int argc, const char **argv, const struct option *options,
                    unsigned int flags, struct parseopt *ctx);

#endif /* LIMB_PARSEOPT_H */
