/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_SHA3_H
#define LIMB_SHA3_H

#include <stddef.h> /* size_t */
#include <limb/int.h>

#define sha3_max_permutation_size 25
#define sha3_max_rate_in_qwords 24

struct sha3_ctx
{
    /* 1600 bits algorithm hashing state */
    u64 hash[sha3_max_permutation_size];
    /* 1536-bit buffer for leftovers */
    u64 message[sha3_max_rate_in_qwords];
    /* count of bytes in the message[] buffer */
    unsigned rest;
    /* size of a message block processed at once */
    unsigned block_size;
};
typedef struct sha3_ctx sha3_ctx;

extern void sha3_224_init(void *ctx);
extern void sha3_256_init(void *ctx);
extern void sha3_384_init(void *ctx);
extern void sha3_512_init(void *ctx);
extern void sha3_update(const void *msg, size_t mlen, void *ctx);
extern void sha3_final(void * restrict md, void *ctx);

#endif /* LIMB_SHA3_H */
