/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_U16_H
#define LIMB_U16_H

#include <skalibs/uint16.h>
#include <limb/int.h>
#include <limb/u64.h>

#define u16_le(u)                   uint16_little(u)
#define u16_be(u)                   uint16_big(u)

#define u16p_le(u)                  uint16_littlep(u)
#define u16p_be(u)                  uint16_bigp(u)
#define u16pa_le(a,l)               uint16_littlen(a,l)
#define u16pa_be(a,l)               uint16_bign(a,l)

#define u16_pack(dst,val)           uint16_pack((char *) (dst), val)
#define u16_unpack(val,sce)         uint16_unpack((const char *) (sce), val)
#define u16_pack_big(dst,val)       uint16_pack_big((char *) (dst), val)
#define u16_unpack_big(val,sce)     uint16_unpack_big((const char *) (sce), val)

#define U16_FMT      6
#define U16_FMTG     7
#define U16_XFMT     5
#define U16_OFMT     7
#define U16_BFMT    17

#define u16_fmt(s,u)                u64_fmt_generic(s, (u), 10, 0, 0)
#define u16_fmtg(s,u)               u64_fmt_generic(s, (u), 10, 3, ',')
#define u16_xfmt(s,u)               u64_fmt_generic(s, (u), 16, 0, 0)
#define u16_ofmt(s,u)               u64_fmt_generic(s, (u),  8, 0, 0)
#define u16_bfmt(s,u)               u64_fmt_generic(s, (u),  2, 0, 0)

#define u16_0fmt(s,u,m)             u64_0fmt_generic(s, (u), 10, (m), '0', 0, 0)
#define u16_0fmtg(s,u,m)            u64_0fmt_generic(s, (u), 10, (m), '0', 3, ',')
#define u16_0xfmt(s,u,m)            u64_0fmt_generic(s, (u), 16, (m), '0', 0, 0)
#define u16_0ofmt(s,u,m)            u64_0fmt_generic(s, (u),  8, (m), '0', 0, 0)
#define u16_0bfmt(s,u,m)            u64_0fmt_generic(s, (u),  2, (m), '0', 0, 0)

#define u16_scan(dst,s)             uint16_scan(s, (dst))
#define u16_xscan(dst,s)            uint16_xscan(s, (dst))
#define u16_oscan(dst,s)            uint16_oscan(s, (dst))
#define u16_bscan(dst,s)            uint16_bscan(s, (dst))

#define u16_scan0(dst,s)            uint160_scan(s, (dst))
#define u16_xscan0(dst,s)           uint160_xscan(s, (dst))
#define u16_oscan0(dst,s)           uint160_oscan(s, (dst))
#define u16_bscan0(dst,s)           uint160_bscan(s, (dst))

#endif /* LIMB_U16_H */
