/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_U32_H
#define LIMB_U32_H

#include <skalibs/uint32.h>
#include <limb/int.h>
#include <limb/u64.h>

#define u32_le(u)                   uint32_little(u)
#define u32_be(u)                   uint32_big(u)

#define u32p_le(u)                  uint32_littlep(u)
#define u32p_be(u)                  uint32_bigp(u)
#define u32pa_le(a,l)               uint32_littlen(a,l)
#define u32pa_be(a,l)               uint32_bign(a,l)

#define u32_pack(dst,val)           uint32_pack((char *) (dst), val)
#define u32_unpack(val,sce)         uint32_unpack((const char *) (sce), val)
#define u32_pack_big(dst,val)       uint32_pack_big((char *) (dst), val)
#define u32_unpack_big(val,sce)     uint32_unpack_big((const char *) (sce), val)

#define U32_FMT     11
#define U32_FMTG    14
#define U32_XFMT     9
#define U32_OFMT    13
#define U32_BFMT    33

#define u32_fmt(s,u)                u64_fmt_generic(s, (u), 10, 0, 0)
#define u32_fmtg(s,u)               u64_fmt_generic(s, (u), 10, 3, ',')
#define u32_xfmt(s,u)               u64_fmt_generic(s, (u), 16, 0, 0)
#define u32_ofmt(s,u)               u64_fmt_generic(s, (u),  8, 0, 0)
#define u32_bfmt(s,u)               u64_fmt_generic(s, (u),  2, 0, 0)

#define u32_0fmt(s,u,m)             u64_0fmt_generic(s, (u), 10, (m), '0', 0, 0)
#define u32_0fmtg(s,u,m)            u64_0fmt_generic(s, (u), 10, (m), '0', 3, ',')
#define u32_0xfmt(s,u,m)            u64_0fmt_generic(s, (u), 16, (m), '0', 0, 0)
#define u32_0ofmt(s,u,m)            u64_0fmt_generic(s, (u),  8, (m), '0', 0, 0)
#define u32_0bfmt(s,u,m)            u64_0fmt_generic(s, (u),  2, (m), '0', 0, 0)

#define u32_scan(dst,s)             uint32_scan(s, (dst))
#define u32_xscan(dst,s)            uint32_xscan(s, (dst))
#define u32_oscan(dst,s)            uint32_oscan(s, (dst))
#define u32_bscan(dst,s)            uint32_bscan(s, (dst))

#define u32_scan0(dst,s)            uint320_scan(s, (dst))
#define u32_xscan0(dst,s)           uint320_xscan(s, (dst))
#define u32_oscan0(dst,s)           uint320_oscan(s, (dst))
#define u32_bscan0(dst,s)           uint320_bscan(s, (dst))

#endif /* LIMB_U32_H */
