/* This file is part of limb                           https://lila.oss/limb
 * Copyright (C) 2023 Olivier Brunel                          jjk@jjacky.com */
/* SPDX-License-Identifier: GPL-2.0-only */
#ifndef LIMB_U64_H
#define LIMB_U64_H

#include <stddef.h> /* size_t */
#include <skalibs/uint64.h>
#include <limb/int.h>

#define u64_le(u)                   uint64_little(u)
#define u64_be(u)                   uint64_big(u)

#define u64p_le(u)                  uint64_littlep(u)
#define u64p_be(u)                  uint64_bigp(u)
#define u64pa_le(a,l)               uint64_littlen(a,l)
#define u64pa_be(a,l)               uint64_bign(a,l)

#define u64_pack(dst,val)           uint64_pack((char *) (dst), val)
#define u64_unpack(val,sce)         uint64_unpack((const char *) (sce), val)
#define u64_pack_big(dst,val)       uint64_pack_big((char *) (dst), val)
#define u64_unpack_big(val,sce)     uint64_unpack_big((const char *) (sce), val)

extern int u64_pack_trim(char *dst, size_t dlen, u64 val);
extern int u64_unpack_trim(u64 *val, const char *sce, size_t slen);

extern size_t u64_fmt_generic(char *s, u64 u, u8 base, u8 grp, const char sep);
extern size_t u64_0fmt_generic(char *s, u64 u, u8 base, size_t min, const char fill,
                               u8 grp, const char sep);

#define U64_FMT     21
#define U64_FMTG    27
#define U64_XFMT    17
#define U64_OFMT    25
#define U64_BFMT    65

#define u64_fmt(s,u)                u64_fmt_generic(s, (u), 10, 0, 0)
#define u64_fmtg(s,u)               u64_fmt_generic(s, (u), 10, 3, ',')
#define u64_xfmt(s,u)               u64_fmt_generic(s, (u), 16, 0, 0)
#define u64_ofmt(s,u)               u64_fmt_generic(s, (u),  8, 0, 0)
#define u64_bfmt(s,u)               u64_fmt_generic(s, (u),  2, 0, 0)

#define u64_0fmt(s,u,m)             u64_0fmt_generic(s, (u), 10, (m), '0', 0, 0)
#define u64_0fmtg(s,u,m)            u64_0fmt_generic(s, (u), 10, (m), '0', 3, ',')
#define u64_0xfmt(s,u,m)            u64_0fmt_generic(s, (u), 16, (m), '0', 0, 0)
#define u64_0ofmt(s,u,m)            u64_0fmt_generic(s, (u),  8, (m), '0', 0, 0)
#define u64_0bfmt(s,u,m)            u64_0fmt_generic(s, (u),  2, (m), '0', 0, 0)

#define u64_scan(dst,s)             uint64_scan(s, (dst))
#define u64_xscan(dst,s)            uint64_xscan(s, (dst))
#define u64_oscan(dst,s)            uint64_oscan(s, (dst))
#define u64_bscan(dst,s)            uint64_bscan(s, (dst))

#define u64_scan0(dst,s)            uint640_scan(s, (dst))
#define u64_xscan0(dst,s)           uint640_xscan(s, (dst))
#define u64_oscan0(dst,s)           uint640_oscan(s, (dst))
#define u64_bscan0(dst,s)           uint640_bscan(s, (dst))

extern int msb64(u64 val);

#endif /* LIMB_U64_H */
